package org.springblade.datasource.transform.oracle;

import cn.hutool.core.map.CaseInsensitiveMap;
import com.alibaba.druid.util.JdbcUtils;
import com.google.common.collect.Lists;
import lombok.SneakyThrows;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springblade.datasource.support.DynamicDataSourceSupport;
import org.springblade.datasource.transform.ColumnEnum;
import org.springblade.datasource.transform.ColumnInfo;
import org.springblade.datasource.transform.IndexInfo;
import org.springblade.datasource.transform.TableInfo;
import org.springblade.datasource.transform.general.GeneralModel;
import org.springblade.datasource.transform.general.GeneralParseTableHandle;
import org.springblade.datasource.transform.mysql.SQLColumnTypeEnum;
import org.springblade.datasource.transform.util.ListUtils;
import org.springblade.etl.entity.ReportDatasource;

import java.sql.*;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.stream.Collectors;

/**
 * @author 喜欢番茄火锅
 * @date 2023/8/28 10:24
 */
@Slf4j
public class OracleParseTableHandle {
    /**
     * 将源信息转换成通用表数据
     *
     * @param datasource
     * @return {@link TableInfo}
     * @author 王瑞
     * @date 2023/8/16
     */
    @SneakyThrows
    public TableInfo parseTableInfo(ReportDatasource datasource, List<GeneralModel> model, String tableViewName, String tableComment) {
        if (datasource == null) {
            return null;
        }
        Connection connection = DynamicDataSourceSupport.createConnection(datasource);
        try {
            DatabaseMetaData metaData = connection.getMetaData();
            String schemaPattern = datasource.getDbSchema();
            TableInfo tableInfo = new TableInfo();
            tableInfo.setTableName(tableViewName);
            if (StringUtils.isNotBlank(tableComment)) {
                tableInfo.setTableComment(tableComment);
            } else {
                getTables(metaData, schemaPattern, tableViewName, tableInfo);
            }

            tableInfo.setColumnInfo(getColumnInfo(tableViewName, metaData, schemaPattern, model));
            //todo Oracle 索引解析
//            tableInfo.setIndexInfo(getIndexInfo(tableViewName, metaData, schemaPattern));

            return tableInfo;
        } finally {
            JdbcUtils.close(connection);
        }
    }

    @SneakyThrows
    public TableInfo getTables(DatabaseMetaData metaData, String schema, String tableViewName, TableInfo tableInfo) {
        ResultSet rs = metaData.getTables(null, schema, tableViewName, new String[]{"TABLE", "VIEW"});
        while (rs.next()) {
            tableInfo.setTableComment(rs.getString("REMARKS"));
        }
        rs.close();

        return tableInfo;
    }

    private List<IndexInfo> getIndexInfo(String tableViewName, DatabaseMetaData metaData, String schemaPattern) throws SQLException {
        ResultSet rs = null;
        List<IndexInfo> list = new ArrayList<>();
        HashMap<String, IndexInfo> map = new HashMap<>();

        rs = metaData.getIndexInfo(null, schemaPattern, tableViewName, false, false);
        ResultSetMetaData md2 = rs.getMetaData();
        while (rs.next()) {

            String indexName = rs.getString("INDEX_NAME");

            IndexInfo indexInfo = map.get(indexName);
            if (indexInfo != null) {
                List<IndexInfo.IndexColumn> indexColumn = indexInfo.getIndexColumn();
                indexColumn(rs, indexInfo, indexColumn);
                continue;
            }
            indexInfo = new IndexInfo();
            if ("PRIMARY".equals(indexName)) {
                indexInfo.setPrimaryKey(true);
            }
            indexInfo.setIndexName(indexName);
            indexInfo.setNonUnique(rs.getBoolean("NON_UNIQUE"));

            ArrayList<IndexInfo.IndexColumn> indexColumns = Lists.newArrayList();
            indexColumn(rs, indexInfo, indexColumns);
            indexInfo.setIndexColumn(indexColumns);
            map.put(indexName, indexInfo);

        }
        return map.keySet().stream().map(map::get).collect(Collectors.toList());
    }

    private void indexColumn(ResultSet rs, IndexInfo indexInfo, List<IndexInfo.IndexColumn> indexColumn) throws SQLException {
        IndexInfo.IndexColumn indexColumn2 = indexInfo.new IndexColumn();
        indexColumn2.setColumnName(rs.getString("COLUMN_NAME"));
        indexColumn2.setOrdinalPosition(rs.getInt("ORDINAL_POSITION"));
        indexColumn2.setAscOrDesc(rs.getString("ASC_OR_DESC"));

        indexColumn.add(indexColumn2);
    }


    private List<ColumnInfo> getColumnInfo(String tableViewName, DatabaseMetaData metaData, String schemaPattern, List<GeneralModel> model) throws SQLException {

        CaseInsensitiveMap<String, ColumnInfo> modelMap = ListUtils.GeneralModelListToMap(model);

        List<ColumnInfo> sourceColumn = getSourceColumn(tableViewName, metaData, schemaPattern, modelMap);
        CaseInsensitiveMap<String, ColumnInfo> caseInsensitiveMap = ListUtils.ColumnInfoListToMap(sourceColumn);

        List<ColumnInfo> result = new ArrayList<>();
        GeneralParseTableHandle handle = new GeneralParseTableHandle();
        model.stream().forEach(m -> {
            if (m.getSourceColumn()) {

                ColumnInfo columnInfo = caseInsensitiveMap.get(m.getColumnName());
                if (columnInfo == null) {
                    return;
                }
                if (StringUtils.isNotBlank(m.getColumnComments())) {
                    columnInfo.setColumnComments(m.getColumnComments());
                }
                columnInfo.setColumnName(m.getDestColumnName());
                result.add(columnInfo);
                return;

            } else {
                result.add(handle.getColumnInfo(m));
            }
        });
        return result;
    }

    public List<ColumnInfo> getSourceColumn(String tableViewName, DatabaseMetaData metaData, String schemaPattern) throws SQLException {
        ResultSet rs = metaData.getColumns(null, schemaPattern, tableViewName, "%");
        List<ColumnInfo> sourceCloumn = new ArrayList<>();
        while (rs.next()) {
            ColumnInfo columnInfo = new ColumnInfo();

            columnInfo.setColumnComments(rs.getString("REMARKS"));
            columnInfo.setDataType(rs.getString("TYPE_NAME"));
            columnInfo.setColumnName(rs.getString("COLUMN_NAME"));
            columnInfo.setColumnSize(rs.getInt("COLUMN_SIZE"));
            columnInfo.setDecimalDigits(rs.getInt("DECIMAL_DIGITS"));
            if (rs.getString("TYPE_NAME").equalsIgnoreCase("NUMBER")) {
                if (rs.getInt("COLUMN_SIZE") != 0) {
                    if (rs.getInt("COLUMN_SIZE") > 10) {
                        columnInfo.setColumnType(ColumnEnum.LONG);
                    }
                    if (rs.getInt("COLUMN_SIZE") <= 10) {
                        columnInfo.setColumnType(ColumnEnum.INTEGER);
                    }
                } else {
                    columnInfo.setColumnType(ColumnEnum.LONG);
                }
                if (rs.getInt("DECIMAL_DIGITS") != 0) {

                    if (rs.getInt("DECIMAL_DIGITS") == 2) {
                        columnInfo.setColumnType(ColumnEnum.DOUBLE);
                    }
                    if (rs.getInt("DECIMAL_DIGITS") == 1) {
                        columnInfo.setColumnType(ColumnEnum.FLOAT);
                    }
                    if (rs.getInt("DECIMAL_DIGITS") > 2) {
                        columnInfo.setColumnType(ColumnEnum.DECIMAL);
                    }
                }
            } else {
                columnInfo.setColumnType(SQLColumnTypeEnum.resolve(rs.getInt("DATA_TYPE")).getColumnEnum());
            }
            //IS_NULLABLE - YES/NO  NULLABLE - 1(允许为空)/2(不确定)/0(不允许为空)
            columnInfo.setNullAbleStatus("yes".equalsIgnoreCase(rs.getString("IS_NULLABLE")) ? 1 : 0);
            sourceCloumn.add(columnInfo);
        }
        rs.close();
        return sourceCloumn;
    }

    public List<ColumnInfo> getSourceColumn(String tableViewName, DatabaseMetaData metaData, String schemaPattern, CaseInsensitiveMap<String, ColumnInfo> modelMap) throws SQLException {
        ResultSet rs = metaData.getColumns(null, schemaPattern, tableViewName, "%");
        List<ColumnInfo> sourceCloumn = new ArrayList<>();
        while (rs.next()) {
            ColumnInfo columnInfo = new ColumnInfo();
            int column_size = rs.getInt("COLUMN_SIZE");
            int decimal_digits = rs.getInt("DECIMAL_DIGITS");
            String column_name = rs.getString("COLUMN_NAME");
            ColumnInfo userColumnInfo = modelMap.get(column_name);
            if (userColumnInfo == null) {
                continue;
            }
            Integer columnSize = userColumnInfo.getColumnSize();
            Integer decimalDigits = userColumnInfo.getDecimalDigits();

            columnInfo.setColumnComments(rs.getString("REMARKS"));
            columnInfo.setDataType(rs.getString("TYPE_NAME"));
            columnInfo.setColumnName(column_name);
            columnInfo.setColumnSize(columnSize);
            columnInfo.setDecimalDigits(decimalDigits);

            if (rs.getString("TYPE_NAME").equalsIgnoreCase("NUMBER")) {
                if (columnSize != 0) {
                    if (columnSize > 10) {
                        columnInfo.setColumnType(ColumnEnum.LONG);
                    }
                    if (columnSize <= 10) {
                        columnInfo.setColumnType(ColumnEnum.INTEGER);
                    }
                } else {
                    columnInfo.setColumnType(ColumnEnum.LONG);
                }
                if (decimalDigits != 0) {

                    if (decimalDigits == 2) {
                        columnInfo.setColumnType(ColumnEnum.DOUBLE);
                    }
                    if (decimalDigits == 1) {
                        columnInfo.setColumnType(ColumnEnum.FLOAT);
                    }
                    if (decimalDigits > 2) {
                        columnInfo.setColumnType(ColumnEnum.DECIMAL);
                    }
                }
            } else {
                columnInfo.setColumnType(SQLColumnTypeEnum.resolve(rs.getInt("DATA_TYPE")).getColumnEnum());
            }
            //IS_NULLABLE - YES/NO  NULLABLE - 1(允许为空)/2(不确定)/0(不允许为空)
            columnInfo.setNullAbleStatus("yes".equalsIgnoreCase(rs.getString("IS_NULLABLE")) ? 1 : 0);
            sourceCloumn.add(columnInfo);
        }
        rs.close();
        return sourceCloumn;
    }
}
